# Generated by Django 4.2 on 2025-06-17 14:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp', '0018_alter_activity_payload_alter_activity_response_body'),
    ]

    operations = [
        migrations.CreateModel(
            name='SiteSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(default='My App', max_length=200)),
                ('description', models.TextField(blank=True)),
                ('terms_url', models.URLField(blank=True)),
            ],
        ),
    ]
