# Generated by Django 4.2 on 2025-06-17 14:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp', '0019_sitesettings'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='SiteSettings',
            new_name='SiteSetting',
        ),
    ]
